function NavBackboneFLayer(vectorGrid, s) {

	this.navBackboneJSON = s;
	this.vectorGrid = vectorGrid;
	this.featuresLoadCache = {};

	this.loadNavBackbone = function () {
		var backbone = this.navBackboneJSON;
		//console.log("backbone= " + JSON.stringify(backbone));
		
		var points = backbone['points'];
		this.points=points;
		
		var polygons = backbone['polygons'];
		var links = backbone['links'];

		//load points
		for (var key in points) {
			var pointData = points[key];
			pointData['id'] = key;
			var navNodeFeature = this.createNavNodeFeature(pointData);
			this.featuresLoadCache[key] = navNodeFeature;

		}

		//load polygons
		for (var key in polygons) {
			//console.log("pkey= "+ pkey);
			polygonData = polygons[key];
			polygonData['id'] = key;
			var navNodePolygon = this.createNavNodePolygonFeature(polygonData);
			this.featuresLoadCache[key] = navNodePolygon;

			var linkPoints = polygonData["linkPoints"];
			for (var pkey in linkPoints) {
				//console.log("pkey= "+ pkey);
				var linkedNavNode = this.featuresLoadCache[pkey];
				if (!navNodePolygon['linkPoints'])
					navNodePolygon['linkPoints'] = [];
				navNodePolygon['linkPoints'].push(linkedNavNode);
			}
		}

		for (var key in links) {
			var linkData = links[key];
			linkData['id'] = key;
			var navLink = this.createNavLinkFeature(linkData);

			var startPoint = this.featuresLoadCache[linkData['startPoint']];
			navLink['startPoint'] = startPoint;
			if (startPoint.get("type") == "nav-node-bbl") {
				startPoint["connectLink"] = navLink;
			}
			if (!startPoint['links'])
				startPoint['links'] = [];
			startPoint['links'].push(navLink);

			var endPoint = this.featuresLoadCache[linkData['endPoint']];
			navLink['endPoint'] = endPoint;

			if (endPoint.get("type") == "nav-node-bbl") {
				endPoint["connectLink"] = navLink;
			}
			if (!endPoint['links'])
				endPoint['links'] = [];
			endPoint['links'].push(navLink);

			var navNodesBBL = linkData["navNodesBBL"];
			if (navNodesBBL) {
				for (var lkey in navNodesBBL) {
					var destNavNodeFeature = this.featuresLoadCache[lkey];
					if (!navLink['navNodesBBL'])
						navLink['navNodesBBL'] = [];
					navLink['navNodesBBL'].push(destNavNodeFeature);
					destNavNodeFeature['hostLink'] = navLink;
				}
			}

			this.featuresLoadCache[key] = navLink;

		};
	}

	// load nav layer

	this.navNodeStyle = new ol.style.Style({
			image : new ol.style.Circle({
				radius : 9,
				fill : new ol.style.Fill({
					color : 'rgba(0, 255, 0, 0)'
				}),
				stroke : new ol.style.Stroke({
					color : 'rgba(0, 255, 0, 0)',
					width : 2
				})
			})
		});

	this.navNodeBBLPointStyle = new ol.style.Style({
			image : new ol.style.Circle({
				radius : 9,
				fill : new ol.style.Fill({
					color : 'rgba(255, 255, 255, 0)'
				}),
				stroke : new ol.style.Stroke({
					color : 'rgba(0, 255, 0 ,0)',
					width : 2
				})
			})
		});

	this.navPolygonStyle = new ol.style.Style({
			stroke : new ol.style.Stroke({
				color : 'rgba(0, 255, 0, 0)',
				width : 1

			}),
			fill : new ol.style.Fill({
				color : 'rgba(0, 255, 0, 0)'
			})
		});

	this.navConnectionLinkStyle = new ol.style.Style({
			stroke : new ol.style.Stroke({
				color : 'rgba(0, 255, 0,0)',
				width : 6
			})
		});

	this.isNavPolygon = function (feature) {
		return feature && feature.get("type") == "nav-node-polygon";
	}
	this.isLinkBBN = function (feature) {
		return feature && feature.get("type") == "link-bbn";
	}
	this.isNavLinkConnectNode = function (feature) {
		return feature && feature.get("type") == "nav-node-bbl";
	}

	this.createNavNodeFeature = function (pointData) {

		var point = new ol.geom.Point(pointData['coord']);
		var nodeFeature =
			new ol.Feature({
				type : pointData['type'],
				geometry : point,
				name : pointData['label']
			});
		nodeFeature.setId(pointData['id']);
		if (this.isNavLinkConnectNode(nodeFeature))
			nodeFeature.setStyle(this.navNodeBBLPointStyle);
		else
			nodeFeature.setStyle(this.navNodeStyle);
		this.vectorGrid.getSource().addFeature(nodeFeature);
		return nodeFeature;
	}

	this.createNavNodePolygonFeature = function (polygonData) {

		var polygon = new ol.geom.Polygon(polygonData['coord']);
		var nodeFeature =
			new ol.Feature({
				type : 'nav-node-polygon',
				geometry : polygon,
				name : polygonData['label']
			});

		nodeFeature.setId(polygonData['id']);
		nodeFeature.setStyle(this.navPolygonStyle);
		this.vectorGrid.getSource().addFeature(nodeFeature);
		return nodeFeature;
	}

	this.createNavLinkFeature = function (linkData) {

		var linkFeature =
			new ol.Feature({
				type : 'nav-link',
				geometry : new ol.geom.LineString(linkData['coord']),
				name : linkData['label']
			});
		linkFeature.setId(linkData['id']);
		linkFeature.setStyle(this.navConnectionLinkStyle);
		this.vectorGrid.getSource().addFeature(linkFeature);
		return linkFeature;
	}

	this.getSelectedNavFeature = function (coord) {
		//console.log("getSelectedNavFeature= " + coord);
		var destinationPixel = map.getPixelFromCoordinate(coord);
		var selectedFeature = undefined;
		map.forEachFeatureAtPixel(
			destinationPixel,
			function (feature, layer) {
			var type = feature.get('type');
			//console.log("feature at pixel= " + feature.get('type'));
			//console.log("feature= " + feature.get('type'));

			if (!selectedFeature && (type == "nav-node" || type == "nav-link" || type == "nav-node-polygon"))
				selectedFeature = feature;
			else if ( feature.get('type') == "nav-node") {
				selectedFeature = feature;
				//return;
			} else if ( feature.get('type') == "nav-node-polygon") {
				selectedFeature = feature;
			}
		},
			null,

			function (layer) {
			return layer === navigatorCore.navBackboneFLayer.vectorGrid;
			//return true;
		});
		//if (selectedFeature)
			//console.log("selectedFeature= " + selectedFeature.get('type'));
		return selectedFeature;

	}

	this.findClosestFeatureAndPoint = function (coord) {

		var result = {};
		var closest;
		var theClosestFeature;
		var newClosest;
		var closetDistance;
		var newClosestDistance;
		//	console.log("this.vectorGrid= " + this.vectorGrid);
		this.vectorGrid.getSource().forEachFeature(
			function (feature) {
			var type = feature.get('type');
			// var isNav=feature.getId()&& feature.getId().indexOf('temp')==-1;
			// console.log("feature= " + type +' '+ feature.getId());
			if ((type == "nav-node" || type == "nav-link" || type == "nav-node-polygon"))
				if (!closetDistance) {
					closest = feature.getGeometry().getClosestPoint(coord);
					closetDistance = navigatorCore.navBackboneFLayer.getDistanceSqr(coord, closest);
					theClosestFeature = feature;
				} else {
					newClosest = feature.getGeometry().getClosestPoint(coord);
					newClosestDistance = navigatorCore.navBackboneFLayer.getDistanceSqr(coord, newClosest);
					if (newClosestDistance < closetDistance) {
						closetDistance = newClosestDistance;
						closest = newClosest;
						theClosestFeature = feature;
					} else if (newClosestDistance == closetDistance && type == "nav-node-polygon") {
						closetDistance = newClosestDistance;
						closest = newClosest;
						theClosestFeature = feature;

					}
				}
		})
		result['coord'] = closest;
		result['feature'] = theClosestFeature;
		// console.log("closest= " + closest);
		//  console.log(" result['feature']= " +  result['feature']);

		return result;
	}

	this.getClosestNavNodesAndOffset = function (coord,selectedFeature) {
		var result = {};
    if(!selectedFeature) return result
		if ( selectedFeature && (selectedFeature.get('type') == "nav-node" || selectedFeature.get('type') == "nav-node-bbl")) {
			result[selectedFeature.getId()] = 0;
			return result;
		} else if (selectedFeature && selectedFeature.get('type') == "nav-node-polygon") {
			var linkPoints = selectedFeature["linkPoints"];
			for (var pkey in linkPoints) {
				//console.log("pkey= "+ pkey);
				var point = linkPoints[pkey]
				var coord2 = point.getGeometry().getCoordinates();
				var offset = this.getDistance(coord,coord2)
				result[point.getId()] = offset;
			}

			return result;

		} else if (selectedFeature.get('type') == "nav-link") {
			

	
			var startNode = selectedFeature['startPoint'];
			var coordStart = startNode.getGeometry().getCoordinates();
			var offsetStart = this.getDistance(coord,coordStart)
			result[startNode.getId()] = offsetStart;
			var endNode = selectedFeature['endPoint'];
			var coordEnd = endNode.getGeometry().getCoordinates();
			var offsetEnd = this.getDistance(coord,coordEnd)
			result[endNode.getId()] = offsetEnd;
			
			if (selectedFeature['navNodesBBL']){
				for (var nkey in selectedFeature['navNodesBBL']) {
				//console.log("pkey= "+ pkey);
				var nodeBBL =selectedFeature['navNodesBBL'][nkey];
				var coordEnd = nodeBBL.getGeometry().getCoordinates();
			    var offsetEndBBL = this.getDistance(coord,coordEnd)
				result[nodeBBL.getId()] = offsetEnd;
				
			  }
			}
			
			return result;

		}

		return null;

	}
	
this.getDistance = function(coord1,coord2){
	return  Math.sqrt(Math.pow(coord1[0] - coord2[0], 2) + Math.pow(coord1[1] - coord2[1], 2));
}

this.getDistanceSqr = function(coord1,coord2){
	return  Math.pow(coord1[0] - coord2[0], 2) + Math.pow(coord1[1] - coord2[1], 2);
}

}